@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -24,31 +24,45 @@ import java.io.IOException;
 import java.io.InputStreamReader;
 import java.text.ParseException;
 
-import junit.framework.TestCase;
-
+import org.apache.ivy.TestHelper;
 import org.apache.ivy.core.module.descriptor.ModuleDescriptor;
 import org.apache.ivy.core.settings.IvySettings;
 import org.apache.ivy.plugins.parser.xml.XmlModuleDescriptorParser;
 import org.apache.ivy.util.DefaultMessageLogger;
 import org.apache.ivy.util.FileUtil;
 import org.apache.ivy.util.Message;
+
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.Delete;
 import org.apache.tools.ant.taskdefs.Echo;
 
-public class IvyPublishTest extends TestCase {
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Rule;
+import org.junit.Test;
+import org.junit.rules.ExpectedException;
+
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertEquals;
+
+public class IvyPublishTest {
     private File cache;
 
     private IvyPublish publish;
 
     private Project project;
 
-    protected void setUp() throws Exception {
+    @Rule
+    public ExpectedException expExc = ExpectedException.none();
+
+    @Before
+    public void setUp() {
         cleanTestDir();
         cleanRep();
         createCache();
-        project = new Project();
+        project = TestHelper.newProject();
         project.init();
         project.setProperty("ivy.settings.file", "test/repositories/ivysettings.xml");
         project.setProperty("build", "build/test/publish");
@@ -65,7 +79,8 @@ public class IvyPublishTest extends TestCase {
         cache.mkdirs();
     }
 
-    protected void tearDown() throws Exception {
+    @After
+    public void tearDown() {
         cleanCache();
         cleanTestDir();
         cleanRep();
@@ -87,9 +102,10 @@ public class IvyPublishTest extends TestCase {
         del.execute();
     }
 
-    public void testMergeParent() throws IOException, ParseException {
-        //publish the parent descriptor first, so that it can be found while
-        //we are reading the child descriptor.
+    @Test
+    public void testMergeParent() throws IOException {
+        // publish the parent descriptor first, so that it can be found while
+        // we are reading the child descriptor.
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -102,9 +118,10 @@ public class IvyPublishTest extends TestCase {
         File art = new File("build/test/publish/resolve-simple-1.0.jar");
         FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
         pubParent.execute();
-        
-        //update=true implies merge=true
-        project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
+
+        // update=true implies merge=true
+        project.setProperty("ivy.dep.file",
+            "test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
         publish.setResolver("1");
         publish.setUpdate(true);
         publish.setOrganisation("apache");
@@ -118,37 +135,128 @@ public class IvyPublishTest extends TestCase {
         // should have published the files with "1" resolver
         File published = new File("test/repositories/1/apache/resolve-extends/ivys/ivy-1.2.xml");
         assertTrue(published.exists());
+        checkPublishedFile(published, "ivy-extends-merged.xml");
+    }
 
+    private void checkPublishedFile(File published, String expectedFilename) throws IOException {
         // do a text compare, since we want to test comments as well as structure.
         // we could do a better job of this with xmlunit
+        try (BufferedReader merged = new BufferedReader(new FileReader(published))) {
+            try (BufferedReader expected = new BufferedReader(new InputStreamReader(getClass()
+                    .getResourceAsStream(expectedFilename)))) {
+                int lineNo = 1;
+                String mergeLine = merged.readLine();
+                String expectedLine = expected.readLine();
+                while (mergeLine != null && expectedLine != null) {
 
-        int lineNo = 1;
+                    // strip timestamps for the comparison
+                    if (mergeLine.contains("<info")) {
+                        mergeLine = mergeLine.replaceFirst("\\s?publication=\"\\d+\"", "");
+                    }
+                    // discard whitespace-only lines
+                    if (!(mergeLine.trim().isEmpty() && expectedLine.trim().isEmpty())) {
+                        assertEquals("published descriptor matches at line[" + lineNo + "]", expectedLine,
+                                mergeLine);
+                    }
 
-        BufferedReader merged = new BufferedReader(new FileReader(published));
-        BufferedReader expected = new BufferedReader(new InputStreamReader(getClass()
-            .getResourceAsStream("ivy-extends-merged.xml")));
-        for (String mergeLine = merged.readLine(),
-                    expectedLine = expected.readLine(); 
-            mergeLine != null && expectedLine != null; 
-            mergeLine = merged.readLine(),
-            expectedLine = expected.readLine()) {
-
-            //strip timestamps for the comparison
-            if (mergeLine.indexOf("<info") >= 0) {
-                mergeLine = mergeLine.replaceFirst("\\s?publication=\"\\d+\"", "");
+                    ++lineNo;
+                    mergeLine = merged.readLine();
+                    expectedLine = expected.readLine();
+                }
             }
-            //discard whitespace-only lines
-            if (!(mergeLine.trim().equals("") && expectedLine.trim().equals(""))) {
-                assertEquals("published descriptor matches at line[" + lineNo + "]", expectedLine, mergeLine);
-            }
-
-            ++lineNo;
         }
     }
-    
-    public void testMinimalMerge() throws IOException, ParseException {
-        //publish the parent descriptor first, so that it can be found while
-        //we are reading the child descriptor.
+
+    /**
+     * Test case for IVY-1248.
+     *
+     * @throws IOException if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1248">IVY-1248</a>
+     */
+    @Test
+    public void testMergeParentWithoutPublishingParent() throws IOException {
+        // here we directly publish a module extending ivy-multiconf.xml,
+        // the module parent is not published not yet in cache
+        // update=true implies merge=true
+        // project.setProperty("ivy.dep.file",
+        // "test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
+        publish.setResolver("1");
+        publish.setUpdate(true);
+        publish.setOrganisation("apache");
+        publish.setModule("resolve-extends");
+        publish.setRevision("1.0");
+        publish.setPubrevision("1.2");
+        publish.setStatus("release");
+        publish.addArtifactspattern("test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
+        publish.execute();
+
+        // should have published the files with "1" resolver
+        File published = new File("test/repositories/1/apache/resolve-extends/ivys/ivy-1.2.xml");
+        assertTrue(published.exists());
+        checkPublishedFile(published, "ivy-extends-merged.xml");
+    }
+
+    /**
+     * Test case for IVY-1248.
+     * Here we directly publish a module extending ivy-multiconf.xml,
+     * the module parent is not published not yet in cache.
+     *
+     * @throws IOException if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1248">IVY-1248</a>
+     */
+    @Test
+    public void testMergeParentWithoutPublishingParentForceDeliver() throws IOException {
+        IvyResolve resolve = new IvyResolve();
+        resolve.setProject(project);
+        resolve.setFile(new File("test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml"));
+        resolve.execute();
+
+        // update=true implies merge=true
+        publish.setResolver("1");
+        publish.setUpdate(true);
+        publish.setOrganisation("apache");
+        publish.setModule("resolve-extends");
+        publish.setRevision("1.0");
+        publish.setPubrevision("1.2");
+        publish.setStatus("release");
+        publish.addArtifactspattern("test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
+        publish.setForcedeliver(true);
+        publish.setSrcivypattern("test/repositories/1/apache/resolve-extends/ivys/ivy-deliver-1.2.xml");
+        publish.execute();
+
+        // should have published the files with "1" resolver
+        File published = new File("test/repositories/1/apache/resolve-extends/ivys/ivy-1.2.xml");
+        assertTrue(published.exists());
+        checkPublishedFile(published, "ivy-extends-merged.xml");
+    }
+
+    @Test
+    public void testMergeParentWithoutLocationAttribute() throws IOException {
+        IvyResolve resolve = new IvyResolve();
+        resolve.setProject(project);
+        resolve.setFile(new File("test/java/org/apache/ivy/ant/extends/child1/ivy-child1.xml"));
+        resolve.execute();
+
+        // update=true implies merge=true
+        // project.setProperty("ivy.dep.file",
+        // "test/java/org/apache/ivy/ant/ivy-extends-multiconf.xml");
+        publish.setResolver("1");
+        publish.setUpdate(true);
+        publish.setPubrevision("1.2");
+        publish.setStatus("release");
+        publish.addArtifactspattern("${build}/[artifact].[ext]");
+        publish.execute();
+
+        // should have published the files with "1" resolver
+        File published = new File("test/repositories/1/apache/child1/ivys/ivy-1.2.xml");
+        assertTrue(published.exists());
+        checkPublishedFile(published, "extends/child1/ivy-child1-merged.xml");
+    }
+
+    @Test
+    public void testMinimalMerge() throws IOException {
+        // publish the parent descriptor first, so that it can be found while
+        // we are reading the child descriptor.
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -162,7 +270,7 @@ public class IvyPublishTest extends TestCase {
         FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
         pubParent.execute();
 
-        //update=true implies merge=true
+        // update=true implies merge=true
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-extends-minimal.xml");
         publish.setResolver("1");
         publish.setUpdate(true);
@@ -177,35 +285,94 @@ public class IvyPublishTest extends TestCase {
         // should have published the files with "1" resolver
         File published = new File("test/repositories/1/apache/resolve-minimal/ivys/ivy-1.2.xml");
         assertTrue(published.exists());
-
-        // do a text compare, since we want to test comments as well as structure.
-        // we could do a better job of this with xmlunit
-
-        int lineNo = 1;
-
-        BufferedReader merged = new BufferedReader(new FileReader(published));
-        BufferedReader expected = new BufferedReader(new InputStreamReader(getClass()
-            .getResourceAsStream("ivy-extends-minimal-merged.xml")));
-        for (String mergeLine = merged.readLine(),
-                    expectedLine = expected.readLine();
-            mergeLine != null && expectedLine != null;
-            mergeLine = merged.readLine(),
-            expectedLine = expected.readLine()) {
-
-            //strip timestamps for the comparison
-            if (mergeLine.indexOf("<info") >= 0) {
-                mergeLine = mergeLine.replaceFirst("\\s?publication=\"\\d+\"", "");
-            }
-            //discard whitespace-only lines
-            if (!(mergeLine.trim().equals("") && expectedLine.trim().equals(""))) {
-                assertEquals("published descriptor matches at line[" + lineNo + "]", expectedLine, mergeLine);
-            }
-
-            ++lineNo;
-        }
+        checkPublishedFile(published, "ivy-extends-minimal-merged.xml");
     }
 
-    public void testSimple() throws Exception {
+    @Test
+    public void testMergeExtraAttributes() throws IOException {
+        // publish the parent descriptor first, so that it can be found while
+        // we are reading the child descriptor.
+        project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
+        IvyResolve res = new IvyResolve();
+        res.setProject(project);
+        res.execute();
+
+        IvyPublish pubParent = new IvyPublish();
+        pubParent.setProject(project);
+        pubParent.setResolver("1");
+        pubParent.setPubrevision("1.0");
+        File art = new File("build/test/publish/resolve-simple-1.0.jar");
+        FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
+        pubParent.execute();
+
+        // update=true implies merge=true
+        project.setProperty("ivy.dep.file",
+            "test/java/org/apache/ivy/ant/ivy-extends-extra-attributes.xml");
+        res = new IvyResolve();
+        res.setProject(project);
+        res.execute();
+
+        publish.setResolver("1");
+        publish.setUpdate(true);
+        publish.setOrganisation("apache");
+        publish.setModule("resolve-minimal");
+        publish.setRevision("1.0");
+        publish.setPubrevision("1.2");
+        publish.setStatus("release");
+        publish.addArtifactspattern("build/test/publish/ivy-extends-extra-attributes.xml");
+        publish.setForcedeliver(true);
+        publish.execute();
+
+        // should have published the files with "1" resolver
+        File published = new File("test/repositories/1/apache/resolve-minimal/ivys/ivy-1.2.xml");
+        assertTrue(published.exists());
+        checkPublishedFile(published, "ivy-extends-extra-attributes-merged.xml");
+    }
+
+    @Test
+    public void testMergeExtraAttributesFromParent() throws IOException {
+        // publish the parent descriptor first, so that it can be found while
+        // we are reading the child descriptor.
+        project.setProperty("ivy.dep.file",
+            "test/java/org/apache/ivy/ant/ivy-multiconf-extra-attributes.xml");
+        IvyResolve res = new IvyResolve();
+        res.setProject(project);
+        res.execute();
+
+        IvyPublish pubParent = new IvyPublish();
+        pubParent.setProject(project);
+        pubParent.setResolver("1");
+        pubParent.setPubrevision("1.0");
+        File art = new File("build/test/publish/resolve-simple-1.0.jar");
+        FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
+        pubParent.execute();
+
+        // update=true implies merge=true
+        project.setProperty("ivy.dep.file",
+            "test/java/org/apache/ivy/ant/ivy-extends-extra-attributes-parent.xml");
+        res = new IvyResolve();
+        res.setProject(project);
+        res.execute();
+
+        publish.setResolver("1");
+        publish.setUpdate(true);
+        publish.setOrganisation("apache");
+        publish.setModule("resolve-minimal");
+        publish.setRevision("1.0");
+        publish.setPubrevision("1.2");
+        publish.setStatus("release");
+        publish.addArtifactspattern("build/test/publish/ivy-extends-extra-attributes.xml");
+        publish.setForcedeliver(true);
+        publish.execute();
+
+        // should have published the files with "1" resolver
+        File published = new File("test/repositories/1/apache/resolve-minimal/ivys/ivy-1.2.xml");
+        assertTrue(published.exists());
+        checkPublishedFile(published, "ivy-extends-extra-attributes-merged.xml");
+    }
+
+    @Test
+    public void testSimple() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -228,11 +395,18 @@ public class IvyPublishTest extends TestCase {
         // should have updated published ivy version
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
             new IvySettings(),
-            new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml").toURL(), false);
+            new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml").toURI().toURL(),
+            false);
         assertEquals("1.2", md.getModuleRevisionId().getRevision());
     }
 
-    public void testHaltOnMissing() throws Exception {
+    /**
+     * Expected failure: publish with haltonmissing and a missing artifact
+     */
+    @Test
+    public void testHaltOnMissing() {
+        expExc.expect(BuildException.class);
+        expExc.expectMessage("missing artifact apache#resolve-simple;1.2!resolve-simple.jar");
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -243,27 +417,35 @@ public class IvyPublishTest extends TestCase {
         publish.setHaltonmissing(true);
         try {
             publish.execute();
-            fail("publish with haltonmissing and a missing artifact should raise an exception");
-        } catch (BuildException ex) {
-            assertTrue(ex.getMessage().indexOf("missing") != -1);
-            assertTrue(ex.getMessage().indexOf("resolve-simple.jar") != -1);
-            // should have do the ivy delivering
+        } finally {
+            // should have delivered the ivy file
             assertTrue(new File("build/test/publish/ivy-1.2.xml").exists());
-            
+
             // should not have published the files with "1" resolver
-            assertFalse(new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml").exists());
-            assertFalse(new File("test/repositories/1/apache/resolve-simple/jars/resolve-simple-1.2.jar").exists());
+            assertFalse(new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml")
+                    .exists());
+            assertFalse(new File(
+                    "test/repositories/1/apache/resolve-simple/jars/resolve-simple-1.2.jar")
+                    .exists());
         }
     }
 
-    public void testHaltOnMissing2() throws Exception {
+    /**
+     * Expected failure: publish with haltonmissing and a missing artifact
+     *
+     * @throws IOException when copy fails
+     */
+    @Test
+    public void testHaltOnMissing2() throws IOException {
+        expExc.expect(BuildException.class);
+        expExc.expectMessage("missing artifact apache#multi;1.2!multi2.jar");
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-publish-multi.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
         res.execute();
 
         // in this test case one artifact is available, and the other one is missing
-        // since we use a transactional resolver, no file should be published at all 
+        // since we use a transactional resolver, no file should be published at all
         File art = new File("build/test/publish/multi1-1.2.jar");
         FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
         publish.setPubrevision("1.2");
@@ -271,12 +453,8 @@ public class IvyPublishTest extends TestCase {
         publish.setHaltonmissing(true);
         try {
             publish.execute();
-            fail("publish with haltonmissing and a missing artifact should raise an exception");
-        } catch (BuildException ex) {
-            assertTrue(ex.getMessage().indexOf("missing") != -1);
-            assertTrue(ex.getMessage().indexOf("multi2.jar") != -1);
-
-            // should have do the ivy delivering
+        } finally {
+            // should have delivered the ivy file
             assertTrue(new File("build/test/publish/ivy-1.2.xml").exists());
 
             // should not have published the files with "transactional" resolver
@@ -284,14 +462,22 @@ public class IvyPublishTest extends TestCase {
         }
     }
 
-    public void testHaltOnMissing3() throws Exception {
+    /**
+     * Expected failure: publish with haltonmissing and a missing artifact
+     *
+     * @throws IOException when copy fails
+     */
+    @Test
+    public void testHaltOnMissing3() throws IOException {
+        expExc.expect(BuildException.class);
+        expExc.expectMessage("missing artifact apache#multi;1.2!multi2.jar");
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-publish-multi.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
         res.execute();
 
         // in this test case one artifact is available, and the other one is missing
-        // this should be detected early and no file should be published at all 
+        // this should be detected early and no file should be published at all
         File art = new File("build/test/publish/multi1-1.2.jar");
         FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
         publish.setPubrevision("1.2");
@@ -299,12 +485,8 @@ public class IvyPublishTest extends TestCase {
         publish.setHaltonmissing(true);
         try {
             publish.execute();
-            fail("publish with haltonmissing and a missing artifact should raise an exception");
-        } catch (BuildException ex) {
-            assertTrue(ex.getMessage().indexOf("missing") != -1);
-            assertTrue(ex.getMessage().indexOf("multi2.jar") != -1);
-
-            // should have do the ivy delivering
+        } finally {
+            // should have delivered the ivy file
             assertTrue(new File("build/test/publish/ivy-1.2.xml").exists());
 
             // should not have published the files with "transactional" resolver
@@ -312,7 +494,8 @@ public class IvyPublishTest extends TestCase {
         }
     }
 
-    public void testPublishNotAllConfigs() throws Exception {
+    @Test
+    public void testPublishNotAllConfigs() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-multiconf.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -337,7 +520,8 @@ public class IvyPublishTest extends TestCase {
         // should have updated published ivy version
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
             new IvySettings(),
-            new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml").toURL(), false);
+            new File("test/repositories/1/apache/resolve-simple/ivys/ivy-1.2.xml").toURI().toURL(),
+            false);
         assertEquals("1.2", md.getModuleRevisionId().getRevision());
 
         // should only contain the default configuration
@@ -346,7 +530,8 @@ public class IvyPublishTest extends TestCase {
         assertEquals("Compile configuration not present", "compile", configs[0]);
     }
 
-    public void testMultiPatterns() throws Exception {
+    @Test
+    public void testMultiPatterns() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-publish-multi.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -371,7 +556,8 @@ public class IvyPublishTest extends TestCase {
         assertTrue(new File("test/repositories/1/apache/multi/jars/multi2-1.2.jar").exists());
     }
 
-    public void testPublishPublicConfigsByWildcard() throws Exception {
+    @Test
+    public void testPublishPublicConfigsByWildcard() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-publish-public.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -387,13 +573,15 @@ public class IvyPublishTest extends TestCase {
 
         // should have do the ivy delivering
         assertTrue(new File("build/test/publish/ivy-1.2.xml").exists());
-        
+
         // should have published the files with "1" resolver
         assertTrue(new File("test/repositories/1/apache/publish-public/ivys/ivy-1.2.xml").exists());
-        assertTrue(new File("test/repositories/1/apache/publish-public/jars/publish-public-1.2.jar").exists());
+        assertTrue(new File("test/repositories/1/apache/publish-public/jars/publish-public-1.2.jar")
+                .exists());
     }
 
-    public void testCustom() throws Exception {
+    @Test
+    public void testCustom() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-custom.xml");
         IvyResolve res = new IvyResolve();
         res.setValidate(false);
@@ -419,7 +607,7 @@ public class IvyPublishTest extends TestCase {
 
         // should have updated published ivy version
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
-            new IvySettings(), dest.toURL(), false);
+            new IvySettings(), dest.toURI().toURL(), false);
         assertEquals("1.2", md.getModuleRevisionId().getRevision());
 
         // should have kept custom attributes
@@ -435,7 +623,8 @@ public class IvyPublishTest extends TestCase {
 
     }
 
-    public void testNoDeliver() throws Exception {
+    @Test
+    public void testNoDeliver() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-latest.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -460,7 +649,8 @@ public class IvyPublishTest extends TestCase {
         // the published ivy version should be ok (ok in ivy-publish file)
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
             new IvySettings(),
-            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURL(), false);
+            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURI().toURL(),
+            false);
         assertEquals("1.3", md.getModuleRevisionId().getRevision());
 
         // should not have done delivery (replace dynamic revisions with static ones)
@@ -468,7 +658,8 @@ public class IvyPublishTest extends TestCase {
                 .getRevision());
     }
 
-    public void testNoDeliverWithBranch() throws Exception {
+    @Test
+    public void testNoDeliverWithBranch() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-latest.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -495,7 +686,8 @@ public class IvyPublishTest extends TestCase {
         // the published ivy version should be ok (ok in ivy-publish file)
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
             new IvySettings(),
-            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURL(), false);
+            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURI().toURL(),
+            false);
         assertEquals("BRANCH1", md.getModuleRevisionId().getBranch());
         assertEquals("1.3", md.getModuleRevisionId().getRevision());
 
@@ -504,7 +696,8 @@ public class IvyPublishTest extends TestCase {
                 .getRevision());
     }
 
-    public void testForceDeliver() throws Exception {
+    @Test
+    public void testForceDeliver() throws IOException, ParseException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-latest.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -530,11 +723,18 @@ public class IvyPublishTest extends TestCase {
         // should have updated published ivy version
         ModuleDescriptor md = XmlModuleDescriptorParser.getInstance().parseDescriptor(
             new IvySettings(),
-            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURL(), false);
+            new File("test/repositories/1/apache/resolve-latest/ivys/ivy-1.3.xml").toURI().toURL(),
+            false);
         assertEquals("1.3", md.getModuleRevisionId().getRevision());
     }
 
-    public void testBadNoDeliver() throws Exception {
+    /**
+     * Test must not publish ivy file with bad revision.
+     *
+     * @throws IOException when copy fails
+     */
+    @Test(expected = BuildException.class)
+    public void testBadNoDeliver() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-latest.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -549,15 +749,12 @@ public class IvyPublishTest extends TestCase {
 
         File art = new File("build/test/publish/resolve-latest-1.3.jar");
         FileUtil.copy(new File("test/repositories/1/org1/mod1.1/jars/mod1.1-1.0.jar"), art, null);
-        try {
-            publish.execute();
-            fail("shouldn't publish ivy file with bad revision");
-        } catch (BuildException ex) {
-            //normal
-        }
+
+        publish.execute();
     }
 
-    public void testReadonly() throws Exception {
+    @Test(expected = BuildException.class)
+    public void testReadonly() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-simple.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -588,8 +785,7 @@ public class IvyPublishTest extends TestCase {
 
         try {
             publish.execute();
-            fail("by default, publish should fail when a readonly artifact already exist");
-        } catch (Exception ex) {
+        } finally {
             assertTrue(dest.exists());
             BufferedReader reader = new BufferedReader(new FileReader(dest));
             assertEquals("old version", reader.readLine());
@@ -597,7 +793,8 @@ public class IvyPublishTest extends TestCase {
         }
     }
 
-    public void testOverwrite() throws Exception {
+    @Test
+    public void testOverwrite() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-simple.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
@@ -632,7 +829,8 @@ public class IvyPublishTest extends TestCase {
         reader.close();
     }
 
-    public void testOverwriteReadOnly() throws Exception {
+    @Test
+    public void testOverwriteReadOnly() throws IOException {
         project.setProperty("ivy.dep.file", "test/java/org/apache/ivy/ant/ivy-simple.xml");
         IvyResolve res = new IvyResolve();
         res.setProject(project);
