@@ -64,7 +64,7 @@ public class UpdateSiteLoaderTest {
     @Test
     public void testIvyDE() throws IOException, ParseException, SAXException, URISyntaxException {
         RepoDescriptor site = loader.load(new URI(
-                "https://www.apache.org/dist/ant/ivyde/updatesite/"));
+                "https://archive.apache.org/dist/ant/ivyde/updatesite/"));
         assertTrue(site.getModules().hasNext());
         Iterator<ModuleDescriptorWrapper> it = site.getModules();
         while (it.hasNext()) {
