@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,6 +17,12 @@
  */
 package org.apache.ivy.plugins.parser.xml;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+
 import java.io.BufferedOutputStream;
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
@@ -31,8 +37,7 @@ import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
-import junit.framework.TestCase;
+import java.util.Properties;
 
 import org.apache.ivy.Ivy;
 import org.apache.ivy.core.module.descriptor.Artifact;
@@ -43,10 +48,31 @@ import org.apache.ivy.core.module.id.ModuleRevisionId;
 import org.apache.ivy.core.settings.IvySettings;
 import org.apache.ivy.plugins.repository.BasicResource;
 import org.apache.ivy.util.FileUtil;
-import org.xml.sax.SAXParseException;
+import org.apache.ivy.util.XMLHelper;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 
-public class XmlModuleUpdaterTest extends TestCase {
+public class XmlModuleUpdaterTest {
 
+    // Java8 tries to load ivy.xsd mentioned in
+    // xsd:noNamespaceLocation via https bypassing our UriResolver so
+    // we must allow using http(s) (and thus all) and external entities
+    private final Properties p = (Properties) System.getProperties().clone();
+
+    @Before
+    public void setUp() {
+        System.setProperty(XMLHelper.ALLOW_DOCTYPE_PROCESSING, "true");
+        System.setProperty(XMLHelper.EXTERNAL_RESOURCES, "all");
+    }
+
+    @After
+    public void tearDown() {
+        XmlModuleDescriptorUpdater.LINE_SEPARATOR = System.lineSeparator();
+        System.setProperties(p);
+    }
+
+    @Test
     public void testUpdate() throws Exception {
         /*
          * For updated file to be equals to updated.xml, we have to fix the line separator to the
@@ -56,17 +82,18 @@ public class XmlModuleUpdaterTest extends TestCase {
         XmlModuleDescriptorUpdater.LINE_SEPARATOR = "\n";
         File dest = new File("build/updated-test.xml");
         dest.deleteOnExit();
-        Map resolvedRevisions = new HashMap();
-        resolvedRevisions.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule2", "branch1", "2+"), "2.5");
-        resolvedRevisions.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule6", "trunk", "latest.integration"), "6.3");
-        
-        Map resolvedBranches = new HashMap();
+        Map<ModuleRevisionId, String> resolvedRevisions = new HashMap<>();
+        resolvedRevisions.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule2", "branch1", "2+"), "2.5");
+        resolvedRevisions.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule6", "trunk", "latest.integration"),
+            "6.3");
+
+        Map<ModuleRevisionId, String> resolvedBranches = new HashMap<>();
         resolvedBranches.put(ModuleRevisionId.newInstance("yourorg", "yourmodule3", "3.1"),
-            "branch1");        
-        resolvedBranches.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule2", "branch1", "2+"), null);
+            "branch1");
+        resolvedBranches.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule2", "branch1", "2+"), null);
 
         GregorianCalendar cal = new GregorianCalendar();
         cal.set(2005, 2, 22, 14, 32, 54);
@@ -74,9 +101,10 @@ public class XmlModuleUpdaterTest extends TestCase {
         Ivy ivy = Ivy.newInstance();
         ivy.setVariable("myvar", "myconf1");
         XmlModuleDescriptorUpdater.update(
-            XmlModuleUpdaterTest.class.getResource("test-update.xml"), dest, 
-            getUpdateOptions(ivy.getSettings(), resolvedRevisions, 
-                "release", "mynewrev", cal.getTime()).setResolvedBranches(resolvedBranches));
+            XmlModuleUpdaterTest.class.getResource("test-update.xml"),
+            dest,
+            getUpdateOptions(ivy.getSettings(), resolvedRevisions, "release", "mynewrev",
+                cal.getTime()).setResolvedBranches(resolvedBranches));
 
         assertTrue(dest.exists());
         String expected = FileUtil.readEntirely(new BufferedReader(new InputStreamReader(
@@ -85,11 +113,12 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertEquals(expected, updated);
     }
 
+    @Test
     public void testUpdateWithComments() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-with-comments.xml").toURL();
-        XmlModuleDescriptorUpdater.update(settingsUrl, new BufferedOutputStream(buffer, 1024), 
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-with-comments.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingsUrl, new BufferedOutputStream(buffer, 1024),
             getUpdateOptions("release", "mynewrev"));
 
         XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
@@ -102,6 +131,7 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertEquals(3, dependencies.length);
     }
 
+    @Test
     public void testVariableReplacement() throws Exception {
         /*
          * For updated file to be equals to updated.xml, we have to fix the line separator to the
@@ -111,18 +141,19 @@ public class XmlModuleUpdaterTest extends TestCase {
         XmlModuleDescriptorUpdater.LINE_SEPARATOR = "\n";
         File dest = new File("build/updated-test2.xml");
         dest.deleteOnExit();
-        Map resolvedRevisions = new HashMap();
-        resolvedRevisions.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule2", "branch1", "2+"), "2.5");
-        resolvedRevisions.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule6", "trunk", "latest.integration"), "6.3");
-        
-        Map resolvedBranches = new HashMap();
+        Map<ModuleRevisionId, String> resolvedRevisions = new HashMap<>();
+        resolvedRevisions.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule2", "branch1", "2+"), "2.5");
+        resolvedRevisions.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule6", "trunk", "latest.integration"),
+            "6.3");
+
+        Map<ModuleRevisionId, String> resolvedBranches = new HashMap<>();
         resolvedBranches.put(ModuleRevisionId.newInstance("yourorg", "yourmodule3", "3.1"),
             "branch1");
-        resolvedBranches.put(ModuleRevisionId.newInstance(
-            "yourorg", "yourmodule2", "branch1", "2+"), null);
-        
+        resolvedBranches.put(
+            ModuleRevisionId.newInstance("yourorg", "yourmodule2", "branch1", "2+"), null);
+
         GregorianCalendar cal = new GregorianCalendar();
         cal.set(2005, 2, 22, 14, 32, 54);
 
@@ -168,11 +199,12 @@ public class XmlModuleUpdaterTest extends TestCase {
         ivy.setVariable("all", "all");
         ivy.setVariable("regexp", "regexp");
         ivy.setVariable("theirrev", "1.0, 1.1");
-        
+
         XmlModuleDescriptorUpdater.update(
-            XmlModuleUpdaterTest.class.getResource("test-update-withvar.xml"), dest, 
-            getUpdateOptions(ivy.getSettings(), resolvedRevisions, 
-                "release", "mynewrev", cal.getTime()).setResolvedBranches(resolvedBranches));
+            XmlModuleUpdaterTest.class.getResource("test-update-withvar.xml"),
+            dest,
+            getUpdateOptions(ivy.getSettings(), resolvedRevisions, "release", "mynewrev",
+                cal.getTime()).setResolvedBranches(resolvedBranches));
 
         assertTrue(dest.exists());
         String expected = FileUtil.readEntirely(new BufferedReader(new InputStreamReader(
@@ -181,25 +213,27 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertEquals(expected, updated);
     }
 
+    @Test
     public void testUpdateWithImportedMappingOverride() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-configurations-import4.xml").toURL();
-        XmlModuleDescriptorUpdater.update(settingsUrl, buffer, 
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-configurations-import4.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingsUrl, buffer,
             getUpdateOptions("release", "mynewrev"));
 
         String updatedXml = buffer.toString();
 
         // just make sure that 'confmappingoverride="true"' is declared somewhere in the XML.
-        assertTrue("Updated XML doesn't define the confmappingoverride attribute", updatedXml
-                .indexOf("confmappingoverride=\"true\"") != -1);
+        assertTrue("Updated XML doesn't define the confmappingoverride attribute",
+            updatedXml.contains("confmappingoverride=\"true\""));
     }
 
+    @Test
     public void testUpdateWithExcludeConfigurations1() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-                + "test-update-excludedconfs1.xml").toURL();
-        XmlModuleDescriptorUpdater.update(settingsUrl, buffer, 
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-update-excludedconfs1.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingsUrl, buffer,
             getUpdateOptions("release", "mynewrev").setConfsToExclude(new String[] {"myconf2"}));
 
         XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
@@ -221,30 +255,26 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertNotNull("myconf4 has been removed", updatedMd.getConfiguration("myconf4"));
     }
 
+    @Test(expected = IllegalArgumentException.class)
     public void testUpdateWithExcludeConfigurations2() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingFile = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-update-excludedconfs2.xml").toURL();
-        try {
-            XmlModuleDescriptorUpdater.update(settingFile, buffer,
-                getUpdateOptions("release", "mynewrev")
-                    .setConfsToExclude(new String[] {"myconf2"}));
-            fail("IllegalArgumentException hasn't been thrown");
-        } catch (IllegalArgumentException e) {
-            // this is ok
-        } catch (SAXParseException e) {
-            // this is ok too
-        }
+        URL settingFile = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-update-excludedconfs2.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingFile, buffer, getUpdateOptions("release", "mynewrev")
+                .setConfsToExclude(new String[] {"myconf2"}));
     }
 
+    @Test
     public void testUpdateWithExcludeConfigurations3() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-update-excludedconfs3.xml").toURL();
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-update-excludedconfs3.xml").toURI().toURL();
 
-        XmlModuleDescriptorUpdater.update(settingsUrl, buffer, 
-            getUpdateOptions("release", "mynewrev")
-                .setConfsToExclude(new String[] {"myconf2", "conf2"}));
+        XmlModuleDescriptorUpdater.update(
+            settingsUrl,
+            buffer,
+            getUpdateOptions("release", "mynewrev").setConfsToExclude(
+                new String[] {"myconf2", "conf2"}));
 
         XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
         ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
@@ -267,13 +297,14 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertNotNull("myconf4 has been removed", updatedMd.getConfiguration("myconf4"));
     }
 
+    @Test
     public void testUpdateWithExcludeConfigurations4() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-update-excludedconfs4.xml").toURL();
-        XmlModuleDescriptorUpdater.update(settingsUrl, buffer, 
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-update-excludedconfs4.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingsUrl, buffer,
             getUpdateOptions("release", "mynewrev").setConfsToExclude(new String[] {"myconf2"}));
-        
+
         XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
         ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
             new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
@@ -285,21 +316,20 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertEquals("Number of published artifacts incorrect", 4, artifacts.length);
 
         // test that the correct configuration has been removed
-        for (int i = 0; i < artifacts.length; i++) {
-            Artifact current = artifacts[i];
-            List currentConfs = Arrays.asList(current.getConfigurations());
+        for (Artifact current : artifacts) {
+            List<String> currentConfs = Arrays.asList(current.getConfigurations());
             assertTrue("myconf2 hasn't been removed for artifact " + current.getName(),
                 !currentConfs.contains("myconf2"));
         }
     }
 
+    @Test
     public void testUpdateWithExcludeConfigurations5() throws Exception {
         ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/" 
-            + "test-update-excludedconfs5.xml").toURL();
-        XmlModuleDescriptorUpdater.update(settingsUrl, buffer, 
-            getUpdateOptions("release", "mynewrev")
-                .setConfsToExclude(new String[] {"myconf2"}));
+        URL settingsUrl = new File("test/java/org/apache/ivy/plugins/parser/xml/"
+                + "test-update-excludedconfs5.xml").toURI().toURL();
+        XmlModuleDescriptorUpdater.update(settingsUrl, buffer,
+            getUpdateOptions("release", "mynewrev").setConfsToExclude(new String[] {"myconf2"}));
 
         XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
         ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
@@ -311,27 +341,190 @@ public class XmlModuleUpdaterTest extends TestCase {
         assertEquals("Number of dependencies is incorrect", 8, deps.length);
 
         // check that none of the dependencies contains myconf2
-        for (int i = 0; i < deps.length; i++) {
-            String name = deps[i].getDependencyId().getName();
+        for (DependencyDescriptor dep : deps) {
+            String name = dep.getDependencyId().getName();
             assertFalse("Dependency " + name + " shouldn't have myconf2 as module configuration",
-                Arrays.asList(deps[i].getModuleConfigurations()).contains("myconf2"));
-            assertEquals("Dependency " + name
-                    + " shouldn't have a dependency artifact for configuration myconf2", 0, deps[i]
-                    .getDependencyArtifacts("myconf2").length);
+                Arrays.asList(dep.getModuleConfigurations()).contains("myconf2"));
+            assertEquals(
+                "Dependency " + name
+                        + " shouldn't have a dependency artifact for configuration myconf2",
+                0, dep.getDependencyArtifacts("myconf2").length);
         }
     }
 
-    private UpdateOptions getUpdateOptions(String status, String revision) {
-        return getUpdateOptions(new IvySettings(), new HashMap(), status, revision, new Date());
+    /**
+     * Test case for IVY-1356.
+     *
+     * @throws Exception if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1356">IVY-1356</a>
+     */
+    @Test
+    public void testMergedUpdateWithExtendsAndExcludes() throws Exception {
+        URL url = XmlModuleUpdaterTest.class.getResource("test-extends-dependencies-exclude.xml");
+
+        XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
+        ModuleDescriptor md = parser.parseDescriptor(new IvySettings(), url, true);
+
+        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        XmlModuleDescriptorUpdater.update(url, buffer, getUpdateOptions("release", "mynewrev")
+                .setMerge(true).setMergedDescriptor(md));
+
+        ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
+            new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
+                    false), true);
+
+        DependencyDescriptor[] deps = updatedMd.getDependencies();
+        assertNotNull("Dependencies shouldn't be null", deps);
+        assertEquals("Number of dependencies is incorrect", 2, deps.length);
+
+        // test indentation
+        String updatedXml = buffer.toString();
+        System.out.println(updatedXml);
+        assertTrue(updatedXml.contains(XmlModuleDescriptorUpdater.LINE_SEPARATOR
+                + "\t\t<dependency org=\"myorg\" name=\"mymodule1\" rev=\"1.0\" conf=\"default->default\"/>"
+                + XmlModuleDescriptorUpdater.LINE_SEPARATOR));
     }
 
-    private UpdateOptions getUpdateOptions(IvySettings settings, Map resolvedRevisions, 
-            String status, String revision, Date pubdate) {
-        return new UpdateOptions()
-            .setSettings(settings)
-            .setResolvedRevisions(resolvedRevisions)
-            .setStatus(status)
-            .setRevision(revision)
-            .setPubdate(pubdate);
+    /**
+     * Test case for IVY-1420.
+     *
+     * @throws Exception if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1420">IVY-1420</a>
+     */
+    @Test
+    public void testMergedUpdateWithExtendsAndDefaultConfMappings() throws Exception {
+        URL url = XmlModuleUpdaterTest.class.getResource("test-extends-configurations-defaultconfmapping.xml");
+
+        XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
+        ModuleDescriptor md = parser.parseDescriptor(new IvySettings(), url, true);
+
+        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        XmlModuleDescriptorUpdater.update(url, buffer,
+                getUpdateOptions("release", "mynewrev")
+                        .setMerge(true)
+                        .setMergedDescriptor(md));
+
+        ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
+                new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
+                false), true);
+
+        Configuration[] configurations = updatedMd.getConfigurations();
+        assertNotNull("Configurations shouldn't be null", configurations);
+        assertEquals("Number of configurations is incorrect", 3, configurations.length);
+
+        String updatedXml = buffer.toString();
+        System.out.println(updatedXml);
+        assertTrue(updatedXml.contains("configurations defaultconfmapping=\"conf1,default->@()\""));
+        assertTrue(updatedXml.contains("dependencies defaultconfmapping=\"conf1,default->@()\""));
+    }
+
+    /**
+     * Test case for IVY-1437.
+     *
+     * @throws Exception if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1437">IVY-1437</a>
+     */
+    @Test
+    public void testMergedUpdateWithExtendsAndConfigurationsInheritance() throws Exception {
+        URL url = XmlModuleUpdaterTest.class.getResource("test-extends-configurations-inherit.xml");
+
+        XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
+        ModuleDescriptor md = parser.parseDescriptor(new IvySettings(), url, true);
+
+        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        XmlModuleDescriptorUpdater.update(url, buffer,
+                getUpdateOptions("release", "mynewrev")
+                        .setMerge(true)
+                        .setMergedDescriptor(md));
+
+        ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
+                new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
+                        false), true);
+
+        Configuration[] configurations = updatedMd.getConfigurations();
+        assertNotNull("Configurations shouldn't be null", configurations);
+        assertEquals("Number of configurations is incorrect", 2, configurations.length);
+
+        String updatedXml = buffer.toString();
+        System.out.println(updatedXml);
+        assertTrue(updatedXml.contains("configurations defaultconf=\"compile\" defaultconfmapping=\"*->default\""));
+        assertTrue(updatedXml.contains("dependencies defaultconf=\"compile\" defaultconfmapping=\"*->default\""));
+    }
+
+    /**
+     * Test case for IVY-1315.
+     *
+     * @throws Exception if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1315">IVY-1315</a>
+     */
+    @Test
+    public void testMergedUpdateWithInclude() throws Exception {
+        URL url = XmlModuleUpdaterTest.class.getResource("test-update-excludedconfs6.xml");
+
+        XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
+        ModuleDescriptor md = parser.parseDescriptor(new IvySettings(), url, true);
+
+        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        XmlModuleDescriptorUpdater.update(url, buffer,
+                getUpdateOptions("release", "mynewrev")
+                        .setMerge(true)
+                        .setMergedDescriptor(md));
+
+        ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
+                new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
+                        false), true);
+
+        Configuration[] configurations = updatedMd.getConfigurations();
+        assertNotNull("Configurations shouldn't be null", configurations);
+        assertEquals("Number of configurations is incorrect", 6, configurations.length);
+
+        String updatedXml = buffer.toString();
+        System.out.println(updatedXml);
+        assertTrue(updatedXml.contains("dependencies defaultconf=\"conf1->default\""));
+    }
+
+    /**
+     * Test case for IVY-1419.
+     *
+     * @throws Exception if something goes wrong
+     * @see <a href="https://issues.apache.org/jira/browse/IVY-1419">IVY-1419</a>
+     */
+    @Test
+    public void testMergedUpdateWithIncludeAndExcludedConf() throws Exception {
+        URL url = XmlModuleUpdaterTest.class.getResource("test-update-excludedconfs6.xml");
+
+        XmlModuleDescriptorParser parser = XmlModuleDescriptorParser.getInstance();
+        ModuleDescriptor md = parser.parseDescriptor(new IvySettings(), url, true);
+
+        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        XmlModuleDescriptorUpdater.update(url, buffer,
+                getUpdateOptions("release", "mynewrev")
+                        .setMerge(true)
+                        .setMergedDescriptor(md)
+                        .setConfsToExclude(new String[]{"conf1"}));
+
+        ModuleDescriptor updatedMd = parser.parseDescriptor(new IvySettings(),
+                new ByteArrayInputStream(buffer.toByteArray()), new BasicResource("test", false, 0, 0,
+                        false), true);
+
+        Configuration[] configurations = updatedMd.getConfigurations();
+        assertNotNull("Configurations shouldn't be null", configurations);
+        assertEquals("Number of configurations is incorrect", 5, configurations.length);
+
+        String updatedXml = buffer.toString();
+        System.out.println(updatedXml);
+        assertTrue(updatedXml.contains("dependencies/"));
+    }
+
+    private UpdateOptions getUpdateOptions(String status, String revision) {
+        return getUpdateOptions(new IvySettings(), new HashMap<ModuleRevisionId, String>(), status,
+            revision, new Date());
+    }
+
+    private UpdateOptions getUpdateOptions(IvySettings settings,
+            Map<ModuleRevisionId, String> resolvedRevisions, String status, String revision,
+            Date pubdate) {
+        return new UpdateOptions().setSettings(settings).setResolvedRevisions(resolvedRevisions)
+                .setStatus(status).setRevision(revision).setPubdate(pubdate);
     }
 }
